@@ -2,16 +2,19 @@
 '''
 Manage transport commands via ssh
 '''
+from __future__ import absolute_import
 
 # Import python libs
 import re
 import os
+import sys
+import json
 import time
 import logging
 import subprocess
 
 # Import salt libs
-import salt.exitcodes
+import salt.defaults.exitcodes
 import salt.utils
 import salt.utils.nb_popen
 import salt.utils.vt
@@ -21,6 +24,10 @@ log = logging.getLogger(__name__)
 SSH_PASSWORD_PROMPT_RE = re.compile(r'(?:.*)[Pp]assword(?: for .*)?:', re.M)
 KEY_VALID_RE = re.compile(r'.*\(yes\/no\).*')
 
+# Keep these in sync with ./__init__.py
+RSTR = '_edbc7885e4f9aac9b83b35999b68d015148caf467b78fa39c05f669c0ff89878'
+RSTR_RE = re.compile(r'(?:^|\r?\n)' + RSTR + r'(?:\r?\n|$)')
+
 
 class NoPasswdError(Exception):
     pass
@@ -40,6 +47,22 @@ def gen_key(path):
     subprocess.call(cmd, shell=True)
 
 
+def gen_shell(opts, **kwargs):
+    '''
+    Return the correct shell interface for the target system
+    '''
+    if kwargs['winrm']:
+        try:
+            import saltwinshell
+            shell = saltwinshell.Shell(opts, **kwargs)
+        except ImportError:
+            log.error('The saltwinshell library is not available')
+            sys.exit(salt.defaults.exitcodes.EX_GENERIC)
+    else:
+        shell = Shell(opts, **kwargs)
+    return shell
+
+
 class Shell(object):
     '''
     Create a shell connection object to encapsulate ssh executions
@@ -55,9 +78,15 @@ class Shell(object):
             timeout=None,
             sudo=False,
             tty=False,
-            mods=None):
+            mods=None,
+            identities_only=False,
+            sudo_user=None,
+            remote_port_forwards=None,
+            winrm=False,
+            ssh_options=None):
         self.opts = opts
-        self.host = host
+        # ssh <ipv6>, but scp [<ipv6]:/path
+        self.host = host.strip('[]')
         self.user = user
         self.port = port
         self.passwd = str(passwd) if passwd else passwd
@@ -66,6 +95,9 @@ class Shell(object):
         self.sudo = sudo
         self.tty = tty
         self.mods = mods
+        self.identities_only = identities_only
+        self.remote_port_forwards = remote_port_forwards
+        self.ssh_options = '' if ssh_options is None else ssh_options
 
     def get_error(self, errstr):
         '''
@@ -92,11 +124,12 @@ class Shell(object):
             options.append('PasswordAuthentication=yes')
         else:
             options.append('PasswordAuthentication=no')
-        if self.opts['_ssh_version'] > '4.9':
-            options.append('GSSAPIAuthentication=no')
         options.append('ConnectTimeout={0}'.format(self.timeout))
         if self.opts.get('ignore_host_keys'):
             options.append('StrictHostKeyChecking=no')
+        if self.opts.get('no_host_keys'):
+            options.extend(['StrictHostKeyChecking=no',
+                            'UserKnownHostsFile=/dev/null'])
         known_hosts = self.opts.get('known_hosts_file')
         if known_hosts and os.path.isfile(known_hosts):
             options.append('UserKnownHostsFile={0}'.format(known_hosts))
@@ -106,6 +139,8 @@ class Shell(object):
             options.append('IdentityFile={0}'.format(self.priv))
         if self.user:
             options.append('User={0}'.format(self.user))
+        if self.identities_only:
+            options.append('IdentitiesOnly=yes')
 
         ret = []
         for option in options:
@@ -122,11 +157,14 @@ class Shell(object):
         options = ['ControlMaster=auto',
                    'StrictHostKeyChecking=no',
                    ]
-        if self.opts['_ssh_version'] > '4.9':
+        if self.opts['_ssh_version'] > (4, 9):
             options.append('GSSAPIAuthentication=no')
         options.append('ConnectTimeout={0}'.format(self.timeout))
         if self.opts.get('ignore_host_keys'):
             options.append('StrictHostKeyChecking=no')
+        if self.opts.get('no_host_keys'):
+            options.extend(['StrictHostKeyChecking=no',
+                            'UserKnownHostsFile=/dev/null'])
 
         if self.passwd:
             options.extend(['PasswordAuthentication=yes',
@@ -141,24 +179,31 @@ class Shell(object):
             options.append('Port={0}'.format(self.port))
         if self.user:
             options.append('User={0}'.format(self.user))
+        if self.identities_only:
+            options.append('IdentitiesOnly=yes')
 
         ret = []
         for option in options:
             ret.append('-o {0} '.format(option))
         return ''.join(ret)
 
+    def _ssh_opts(self):
+        return ' '.join(['-o {0}'.format(opt)
+                        for opt in self.ssh_options])
+
     def _copy_id_str_old(self):
         '''
         Return the string to execute ssh-copy-id
         '''
         if self.passwd:
             # Using single quotes prevents shell expansion and
-            # passwords containig '$'
-            return "{0} {1} '{2} -p {3} {4}@{5}'".format(
+            # passwords containing '$'
+            return "{0} {1} '{2} -p {3} {4} {5}@{6}'".format(
                     'ssh-copy-id',
                     '-i {0}.pub'.format(self.priv),
                     self._passwd_opts(),
                     self.port,
+                    self._ssh_opts(),
                     self.user,
                     self.host)
         return None
@@ -170,12 +215,13 @@ class Shell(object):
         '''
         if self.passwd:
             # Using single quotes prevents shell expansion and
-            # passwords containig '$'
-            return "{0} {1} {2} -p {3} {4}@{5}".format(
+            # passwords containing '$'
+            return "{0} {1} {2} -p {3} {4} {5}@{6}".format(
                     'ssh-copy-id',
                     '-i {0}.pub'.format(self.priv),
                     self._passwd_opts(),
                     self.port,
+                    self._ssh_opts(),
                     self.user,
                     self.host)
         return None
@@ -185,7 +231,7 @@ class Shell(object):
         Execute ssh-copy-id to plant the id file on the target
         '''
         stdout, stderr, retcode = self._run_cmd(self._copy_id_str_old())
-        if salt.exitcodes.EX_OK != retcode and stderr.startswith('Usage'):
+        if salt.defaults.exitcodes.EX_OK != retcode and 'Usage' in stderr:
             stdout, stderr, retcode = self._run_cmd(self._copy_id_str_new())
         return stdout, stderr, retcode
 
@@ -197,19 +243,22 @@ class Shell(object):
         # TODO: if tty, then our SSH_SHIM cannot be supplied from STDIN Will
         # need to deliver the SHIM to the remote host and execute it there
 
-        tty = self.tty
-        if ssh != 'ssh':
-            tty = False
-        if self.passwd:
-            opts = self._passwd_opts()
-        if self.priv:
-            opts = self._key_opts()
-        return "{0} {1} {2} {3} {4}".format(
-                ssh,
-                '' if ssh == 'scp' else self.host,
-                '-t -t' if tty else '',
-                opts,
-                cmd)
+        command = [ssh]
+        if ssh != 'scp':
+            command.append(self.host)
+        if self.tty and ssh == 'ssh':
+            command.append('-t -t')
+        if self.passwd or self.priv:
+            command.append(self.priv and self._key_opts() or self._passwd_opts())
+        if ssh != 'scp' and self.remote_port_forwards:
+            command.append(' '.join(['-R {0}'.format(item)
+                                     for item in self.remote_port_forwards.split(',')]))
+        if self.ssh_options:
+            command.append(self._ssh_opts())
+
+        command.append(cmd)
+
+        return ' '.join(command)
 
     def _old_run_cmd(self, cmd):
         '''
@@ -283,16 +332,28 @@ class Shell(object):
         logmsg = 'Executing command: {0}'.format(cmd)
         if self.passwd:
             logmsg = logmsg.replace(self.passwd, ('*' * 6))
-        log.debug(logmsg)
+        if 'decode("base64")' in logmsg or 'base64.b64decode(' in logmsg:
+            log.debug('Executed SHIM command. Command logged to TRACE')
+            log.trace(logmsg)
+        else:
+            log.debug(logmsg)
 
         ret = self._run_cmd(cmd)
         return ret
 
-    def send(self, local, remote):
+    def send(self, local, remote, makedirs=False):
         '''
         scp a file or files to a remote system
         '''
-        cmd = '{0} {1}:{2}'.format(local, self.host, remote)
+        if makedirs:
+            self.exec_cmd('mkdir -p {0}'.format(os.path.dirname(remote)))
+
+        # scp needs [<ipv6}
+        host = self.host
+        if ':' in host:
+            host = '[{0}]'.format(host)
+
+        cmd = '{0} {1}:{2}'.format(local, host, remote)
         cmd = self._cmd_str(cmd, ssh='scp')
 
         logmsg = 'Executing command: {0}'.format(cmd)
@@ -311,58 +372,56 @@ class Shell(object):
                 cmd,
                 shell=True,
                 log_stdout=True,
+                log_stdout_level='trace',
                 log_stderr=True,
+                log_stderr_level='trace',
                 stream_stdout=False,
                 stream_stderr=False)
         sent_passwd = 0
+        send_password = True
         ret_stdout = ''
         ret_stderr = ''
-        while True:
-            stdout, stderr = term.recv()
-            if stdout:
-                ret_stdout += stdout
-            if stderr:
-                ret_stderr += stderr
-            if stdout and SSH_PASSWORD_PROMPT_RE.search(stdout):
-                if len(stdout) > 256:
-                    pass
-                elif not self.passwd:
-                    try:
-                        term.close(terminate=True, kill=True)
-                    except salt.utils.vt.TerminalException:
-                        pass
-                    return '', 'Permission denied, no authentication information', 254
-                if sent_passwd < passwd_retries:
-                    term.sendline(self.passwd)
-                    sent_passwd += 1
-                    continue
+        old_stdout = ''
+
+        try:
+            while term.has_unread_data:
+                stdout, stderr = term.recv()
+                if stdout:
+                    ret_stdout += stdout
+                    buff = old_stdout + stdout
                 else:
-                    # asking for a password, and we can't seem to send it
-                    try:
-                        term.close(terminate=True, kill=True)
-                    except salt.utils.vt.TerminalException:
-                        pass
-                    return '', 'Password authentication failed', 254
-            elif stdout and KEY_VALID_RE.search(stdout):
-                if key_accept:
-                    term.sendline('yes')
-                    continue
-                else:
-                    term.sendline('no')
-                    ret_stdout = ('The host key needs to be accepted, to '
-                                  'auto accept run salt-ssh with the -i '
-                                  'flag:\n{0}').format(stdout)
-                    return ret_stdout, '', 254
-            if not term.isalive():
-                while True:
-                    stdout, stderr = term.recv()
-                    if stdout:
-                        ret_stdout += stdout
-                    if stderr:
-                        ret_stderr += stderr
-                    if stdout is None and stderr is None:
-                        break
-                term.close(terminate=True, kill=True)
-                break
-            time.sleep(0.5)
-        return ret_stdout, ret_stderr, term.exitstatus
+                    buff = stdout
+                if stderr:
+                    ret_stderr += stderr
+                if buff and RSTR_RE.search(buff):
+                    # We're getting results back, don't try to send passwords
+                    send_password = False
+                if buff and SSH_PASSWORD_PROMPT_RE.search(buff) and send_password:
+                    if not self.passwd:
+                        return '', 'Permission denied, no authentication information', 254
+                    if sent_passwd < passwd_retries:
+                        term.sendline(self.passwd)
+                        sent_passwd += 1
+                        continue
+                    else:
+                        # asking for a password, and we can't seem to send it
+                        return '', 'Password authentication failed', 254
+                elif buff and KEY_VALID_RE.search(buff):
+                    if key_accept:
+                        term.sendline('yes')
+                        continue
+                    else:
+                        term.sendline('no')
+                        ret_stdout = ('The host key needs to be accepted, to '
+                                      'auto accept run salt-ssh with the -i '
+                                      'flag:\n{0}').format(stdout)
+                        return ret_stdout, '', 254
+                elif buff and buff.endswith('_||ext_mods||_'):
+                    mods_raw = json.dumps(self.mods, separators=(',', ':')) + '|_E|0|'
+                    term.sendline(mods_raw)
+                if stdout:
+                    old_stdout = stdout
+                time.sleep(0.01)
+            return ret_stdout, ret_stderr, term.exitstatus
+        finally:
+            term.close(terminate=True, kill=True)
