@@ -4,7 +4,6 @@ Manage Dell DRAC.
 
 .. versionadded:: 2015.8.2
 '''
-# pylint: disable=W0141
 
 # Import python libs
 from __future__ import absolute_import
@@ -27,7 +26,7 @@ __proxyenabled__ = ['fx2']
 
 try:
     run_all = __salt__['cmd.run_all']
-except NameError:
+except (NameError, KeyError):
     import salt.modules.cmdmod
     __salt__ = {
         'cmd.run_all': salt.modules.cmdmod._run_all_quiet
@@ -38,7 +37,7 @@ def __virtual__():
     if salt.utils.which('racadm'):
         return True
 
-    return False
+    return (False, 'The drac execution module cannot be loaded: racadm binary not in path.')
 
 
 def __parse_drac(output):
@@ -137,8 +136,13 @@ def __execute_ret(command, host=None,
         for l in cmd['stdout'].splitlines():
             if l.startswith('Security Alert'):
                 continue
+            if l.startswith('RAC1168:'):
+                break
+            if l.startswith('RAC1169:'):
+                break
             if l.startswith('Continuing execution'):
                 continue
+
             if len(l.strip()) == 0:
                 continue
             fmtlines.append(l)
@@ -161,7 +165,8 @@ def get_dns_dracname(host=None,
 
 def set_dns_dracname(name,
                      host=None,
-                     admin_username=None, admin_password=None):
+                     admin_username=None,
+                     admin_password=None):
 
     ret = __execute_ret('set iDRAC.NIC.DNSRacName {0}'.format(name),
                         host=host,
@@ -195,7 +200,7 @@ def system_info(host=None,
     return __parse_drac(cmd['stdout'])
 
 
-def set_niccfg(ip=None, subnet=None, gateway=None, dhcp=False,
+def set_niccfg(ip=None, netmask=None, gateway=None, dhcp=False,
                host=None,
                admin_username=None,
                admin_password=None,
@@ -206,7 +211,7 @@ def set_niccfg(ip=None, subnet=None, gateway=None, dhcp=False,
     if dhcp:
         cmdstr += '-d '
     else:
-        cmdstr += '-s ' + ip + ' ' + subnet + ' ' + gateway
+        cmdstr += '-s ' + ip + ' ' + netmask + ' ' + gateway
 
     return __execute_cmd(cmdstr, host=host,
                          admin_username=admin_username,
@@ -255,10 +260,10 @@ def network_info(host=None,
         cmd['stdout'] = 'Problem getting switch inventory'
         return cmd
 
-    if module not in inv.get('switch'):
+    if module not in inv.get('switch') and module not in inv.get('server'):
         cmd = {}
         cmd['retcode'] = -1
-        cmd['stdout'] = 'No switch {0} found.'.format(module)
+        cmd['stdout'] = 'No module {0} found.'.format(module)
         return cmd
 
     cmd = __execute_ret('getniccfg', host=host,
@@ -542,7 +547,7 @@ def create_user(username, password, permissions,
 
     .. code-block:: bash
 
-        salt dell dracr.create_user [USERNAME] [PASSWORD] [PRIVELEGES]
+        salt dell dracr.create_user [USERNAME] [PASSWORD] [PRIVILEGES]
         salt dell dracr.create_user diana secret login,test_alerts,clear_logs
 
     DRAC Privileges
@@ -610,7 +615,7 @@ def set_permissions(username, permissions,
 
     .. code-block:: bash
 
-        salt dell dracr.set_permissions [USERNAME] [PRIVELEGES]
+        salt dell dracr.set_permissions [USERNAME] [PRIVILEGES]
              [USER INDEX - optional]
         salt dell dracr.set_permissions diana login,test_alerts,clear_logs 4
 
@@ -1319,13 +1324,45 @@ def get_general(cfg_sec, cfg_var, host=None,
         return ret
 
 
-def bare_rac_cmd(cmd, host=None,
-                admin_username=None, admin_password=None):
+def idrac_general(blade_name, command, idrac_password=None,
+                  host=None,
+                  admin_username=None, admin_password=None):
+    '''
+    Run a generic racadm command against a particular
+    blade in a chassis.  Blades are usually named things like
+    'server-1', 'server-2', etc.  If the iDRAC has a different
+    password than the CMC, then you can pass it with the
+    idrac_password kwarg.
 
-    ret = __execute_ret('{0}'.format(cmd),
-                        host=host,
-                        admin_username=admin_username,
-                        admin_password=admin_password)
+    :param blade_name: Name of the blade to run the command on
+    :param command: Command like to pass to racadm
+    :param idrac_password: Password for the iDRAC if different from the CMC
+    :param host: Chassis hostname
+    :param admin_username: CMC username
+    :param admin_password: CMC password
+    :return: stdout if the retcode is 0, otherwise a standard cmd.run_all dictionary
+
+    CLI Example:
+
+    .. code-block:: bash
+
+        salt fx2 chassis.cmd idrac_general server-1 'get BIOS.SysProfileSettings'
+
+    '''
+
+    module_network = network_info(host, admin_username,
+                                  admin_password, blade_name)
+
+    if idrac_password is not None:
+        password = idrac_password
+    else:
+        password = admin_password
+
+    idrac_ip = module_network['Network']['IP Address']
+
+    ret = __execute_ret(command, host=idrac_ip,
+                        admin_username='root',
+                        admin_password=password)
 
     if ret['retcode'] == 0:
         return ret['stdout']
@@ -1354,6 +1391,19 @@ def _update_firmware(cmd,
         return ret
 
 
+def bare_rac_cmd(cmd, host=None,
+                admin_username=None, admin_password=None):
+    ret = __execute_ret('{0}'.format(cmd),
+                        host=host,
+                        admin_username=admin_username,
+                        admin_password=admin_password)
+
+    if ret['retcode'] == 0:
+        return ret['stdout']
+    else:
+        return ret
+
+
 def update_firmware(filename,
                     host=None,
                     admin_username=None,
@@ -1426,3 +1476,5 @@ def update_firmware_nfs_or_cifs(filename, share,
     else:
         raise CommandExecutionError('Unable to find firmware file {0}'
                                     .format(filename))
+
+# def get_idrac_nic()
