@@ -1,5 +1,48 @@
 # -*- coding: utf-8 -*-
+'''
+Module to provide Citrix Netscaler compatibility to Salt (compatible with netscaler 9.2+)
 
+.. versionadded:: 2015.2.0
+
+:depends:
+
+- nsnitro Python module
+
+.. note::
+    You can install nsnitro using:
+
+    .. code-block:: bash
+
+        pip install nsnitro
+
+:configuration: This module accepts connection configuration details either as
+    parameters, or as configuration settings in /etc/salt/minion on the relevant
+    minions
+
+    .. code-block:: yaml
+
+        netscaler.host: 1.2.3.4
+        netscaler.user: user
+        netscaler.pass: password
+
+    This data can also be passed into pillar. Options passed into opts will
+    overwrite options passed into pillar.
+
+:CLI Examples:
+    Calls relying on configuration passed using /etc/salt/minion, grains, or pillars:
+    .. code-block:: bash
+
+        salt-call netscaler.server_exists server_name
+
+    Calls passing configuration as opts
+    .. code-block:: bash
+
+        salt-call netscaler.server_exists server_name netscaler_host=1.2.3.4 netscaler_user=username netscaler_pass=password
+        salt-call netscaler.server_exists server_name netscaler_host=1.2.3.5 netscaler_user=username2 netscaler_pass=password2
+        salt-call netscaler.server_enable server_name2 netscaler_host=1.2.3.5
+        salt-call netscaler.server_up server_name3 netscaler_host=1.2.3.6 netscaler_useSSL=False
+
+'''
 from __future__ import absolute_import
 import logging
 import salt.utils
@@ -26,10 +69,10 @@ def __virtual__():
     Only load this module if the nsnitro library is installed
     '''
     if salt.utils.is_windows():
-        return False
+        return (False, 'The netscaler execution module failed to load: not available on Windows.')
     if HAS_NSNITRO:
         return 'netscaler'
-    return False
+    return (False, 'The netscaler execution module failed to load: the nsnitro python library is not available.')
 
 
 def _connect(**kwargs):
@@ -39,7 +82,7 @@ def _connect(**kwargs):
     connargs = dict()
 
     # Shamelessy ripped from the mysql module
-    def __connarg(name, key=None):
+    def __connarg(name, key=None, default=None):
         '''
         Add key to connargs, only if name exists in our kwargs or as
         netscaler.<name> in __opts__ or __pillar__ Evaluate in said order - kwargs,
@@ -61,14 +104,15 @@ def _connect(**kwargs):
             val = __salt__['config.option']('netscaler.{0}'.format(name), None)
             if val is not None:
                 connargs[key] = val
+            elif default is not None:
+                connargs[key] = default
 
     __connarg('netscaler_host', 'host')
     __connarg('netscaler_user', 'user')
     __connarg('netscaler_pass', 'pass')
-    # useSSL = True will be enforced
-    #_connarg('connection_useSSL', 'useSSL')
+    __connarg('netscaler_useSSL', 'useSSL', True)
 
-    nitro = NSNitro(connargs['host'], connargs['user'], connargs['pass'], True)
+    nitro = NSNitro(connargs['host'], connargs['user'], connargs['pass'], connargs['useSSL'])
     try:
         nitro.login()
     except NSNitroError as error:
