@@ -4,6 +4,7 @@ Module for viewing and modifying OpenBSD sysctl parameters
 '''
 from __future__ import absolute_import
 import os
+import re
 
 # Import salt libs
 import salt.utils
@@ -17,7 +18,10 @@ def __virtual__():
     '''
     Only run on OpenBSD systems
     '''
-    return __virtualname__ if __grains__['os'] == 'OpenBSD' else False
+    if __grains__['os'] == 'OpenBSD':
+        return __virtualname__
+    return (False, 'The openbsd_sysctl execution module cannot be loaded: '
+            'only available on OpenBSD systems.')
 
 
 def show(config_file=False):
@@ -70,7 +74,11 @@ def assign(name, value):
     cmd = 'sysctl {0}="{1}"'.format(name, value)
     data = __salt__['cmd.run_all'](cmd)
 
-    if data['retcode'] != 0:
+    # Certain values cannot be set from this console, at the current
+    # securelevel or there are other restrictions that prevent us
+    # from applying the setting rightaway.
+    if re.match(r'^sysctl:.*: Operation not permitted$', data['stderr']) or \
+      data['retcode'] != 0:
         raise CommandExecutionError('sysctl failed: {0}'.format(
             data['stderr']))
     new_name, new_value = data['stdout'].split(':', 1)
@@ -95,7 +103,8 @@ def persist(name, value, config='/etc/sysctl.conf'):
     # create /etc/sysctl.conf if not present
     if not os.path.isfile(config):
         try:
-            salt.utils.fopen(config, 'w+').close()
+            with salt.utils.fopen(config, 'w+'):
+                pass
         except (IOError, OSError):
             msg = 'Could not create {0}'
             raise CommandExecutionError(msg.format(config))
