@@ -1,6 +1,25 @@
 # -*- coding: utf-8 -*-
 '''
 Package support for OpenBSD
+
+.. note::
+
+    The package repository is configured on each host using ``/etc/pkg.conf``
+
+.. versionchanged:: 2016.3.5
+
+    Package versions on OpenBSD are not normally specified explicitly; instead
+    packages may be available in multiple *flavors*, and *branches* which are
+    specified by the format of the package name. This module allows you to use
+    the same formatting as ``pkg_add(1)``, and will select the empty flavor and
+    default branch by default. Examples:
+
+    .. code-block:: yaml
+
+      - rsync
+      - vim--no_x11
+      - ruby%2.3
+
 '''
 from __future__ import absolute_import
 
@@ -15,14 +34,13 @@ from salt.exceptions import CommandExecutionError, MinionError
 
 log = logging.getLogger(__name__)
 
-
+# FIXME: replace guesswork with `pkg_info -z` to correctly identify package
+#        flavors and branches
 __PKG_RE = re.compile('^((?:[^-]+|-(?![0-9]))+)-([0-9][^-]*)(?:-(.*))?$')
 
 # Define the module's virtual name
 __virtualname__ = 'pkg'
 
-# XXX need a way of setting PKG_PATH instead of inheriting from the environment
-
 
 def __virtual__():
     '''
@@ -30,7 +48,8 @@ def __virtual__():
     '''
     if __grains__['os'] == 'OpenBSD':
         return __virtualname__
-    return False
+    return (False, 'The openbsdpkg execution module cannot be loaded: '
+            'only available on OpenBSD systems.')
 
 
 def list_pkgs(versions_as_list=False, **kwargs):
@@ -79,7 +98,12 @@ def list_pkgs(versions_as_list=False, **kwargs):
 
 def latest_version(*names, **kwargs):
     '''
-    The available version of the package in the repository
+    Return the latest version of the named package available for upgrade or
+    installation. If more than one package name is specified, a dict of
+    name/version pairs is returned.
+
+    If the latest version of a given package is already installed, an empty
+    string will be returned for that package.
 
     CLI Example:
 
@@ -95,29 +119,48 @@ def latest_version(*names, **kwargs):
     for name in names:
         ret[name] = ''
 
-    stems = [x.split('--')[0] for x in names]
-    cmd = 'pkg_info -q -I {0}'.format(' '.join(stems))
-    out = __salt__['cmd.run_stdout'](cmd, python_shell=False, output_loglevel='trace')
-    for line in out.splitlines():
-        try:
-            pkgname, pkgver, flavor = __PKG_RE.match(line).groups()
-        except AttributeError:
-            continue
-        pkgname += '--{0}'.format(flavor) if flavor else ''
-        cur = pkgs.get(pkgname, '')
-        if not cur or __salt__['pkg_resource.compare'](pkg1=cur,
-                                                       oper='<',
-                                                       pkg2=pkgver):
-            ret[pkgname] = pkgver
+        # Query the repository for the package name
+        cmd = 'pkg_info -Q {0}'.format(name)
+        out = __salt__['cmd.run_stdout'](cmd, python_shell=False, output_loglevel='trace')
+
+        # Since we can only query instead of request the specific package
+        # we'll have to go through the returned list and find what we
+        # were looking for.
+        # Keep in mind the match may be flavored.
+        for line in out.splitlines():
+            try:
+                pkgname, pkgver, flavor = __PKG_RE.match(line).groups()
+            except AttributeError:
+                continue
+
+            match = re.match(r'.*\(installed\)$', pkgver)
+            if match:
+                # Package is explicitly marked as installed already,
+                # so skip any further comparison and move on to the
+                # next package to compare (if provided).
+                break
+
+            # First check if we need to look for flavors before
+            # looking at unflavored packages.
+            if "{0}--{1}".format(pkgname, flavor) == name:
+                pkgname += '--{0}'.format(flavor)
+            elif pkgname == name:
+                pass
+            else:
+                # No match just move on.
+                continue
+
+            cur = pkgs.get(pkgname, '')
+            if not cur or salt.utils.compare_versions(ver1=cur,
+                                                      oper='<',
+                                                      ver2=pkgver):
+                ret[pkgname] = pkgver
 
     # Return a string if only one package name passed
     if len(names) == 1:
         return ret[names[0]]
     return ret
 
-# available_version is being deprecated
-available_version = latest_version
-
 
 def version(*names, **kwargs):
     '''
@@ -156,7 +199,8 @@ def install(name=None, pkgs=None, sources=None, **kwargs):
 
         salt '*' pkg.install pkgs='["<package name>", "<package name>"]'
 
-    CLI Example, Install more than one package from a alternate source (e.g. salt file-server, HTTP, FTP, local filesystem):
+    CLI Example, Install more than one package from a alternate source (e.g.
+    salt file-server, HTTP, FTP, local filesystem):
 
     .. code-block:: bash
 
@@ -173,19 +217,37 @@ def install(name=None, pkgs=None, sources=None, **kwargs):
         return {}
 
     old = list_pkgs()
+    errors = []
     for pkg in pkg_params:
+        # A special case for OpenBSD package "branches" is also required in
+        # salt/states/pkg.py
         if pkg_type == 'repository':
-            stem, flavor = (pkg.split('--') + [''])[:2]
-            pkg = '--'.join((stem, flavor))
-        cmd = 'pkg_add -x {0}'.format(pkg)
-        __salt__['cmd.run'](cmd, python_shell=False, output_loglevel='trace')
+            stem, branch = (pkg.split('%') + [''])[:2]
+            base, flavor = (stem.split('--') + [''])[:2]
+            pkg = '{0}--{1}%{2}'.format(base, flavor, branch)
+        cmd = 'pkg_add -x -I {0}'.format(pkg)
+        out = __salt__['cmd.run_all'](
+            cmd,
+            python_shell=False,
+            output_loglevel='trace'
+        )
+        if out['retcode'] != 0 and out['stderr']:
+            errors.append(out['stderr'])
 
     __context__.pop('pkg.list_pkgs', None)
     new = list_pkgs()
-    return salt.utils.compare_dicts(old, new)
+    ret = salt.utils.compare_dicts(old, new)
+
+    if errors:
+        raise CommandExecutionError(
+            'Problem encountered installing package(s)',
+            info={'errors': errors, 'changes': ret}
+        )
+
+    return ret
 
 
-def remove(name=None, pkgs=None, **kwargs):
+def remove(name=None, pkgs=None, purge=False, **kwargs):
     '''
     Remove a single package with pkg_delete
 
@@ -219,17 +281,39 @@ def remove(name=None, pkgs=None, **kwargs):
     if not targets:
         return {}
 
-    cmd = 'pkg_delete -xD dependencies {0}'.format(' '.join(targets))
-    __salt__['cmd.run'](cmd, python_shell=False, output_loglevel='trace')
+    cmd = ['pkg_delete', '-Ix', '-Ddependencies']
+
+    if purge:
+        cmd.append('-cqq')
+
+    cmd.extend(targets)
+
+    out = __salt__['cmd.run_all'](
+        cmd,
+        python_shell=False,
+        output_loglevel='trace'
+    )
+    if out['retcode'] != 0 and out['stderr']:
+        errors = [out['stderr']]
+    else:
+        errors = []
+
     __context__.pop('pkg.list_pkgs', None)
     new = list_pkgs()
-    return salt.utils.compare_dicts(old, new)
+    ret = salt.utils.compare_dicts(old, new)
+
+    if errors:
+        raise CommandExecutionError(
+            'Problem encountered removing package(s)',
+            info={'errors': errors, 'changes': ret}
+        )
+
+    return ret
 
 
 def purge(name=None, pkgs=None, **kwargs):
     '''
-    Package purges are not supported, this function is identical to
-    ``remove()``.
+    Remove a package and extra configuration files.
 
     name
         The name of the package to be deleted.
@@ -254,4 +338,70 @@ def purge(name=None, pkgs=None, **kwargs):
         salt '*' pkg.purge <package1>,<package2>,<package3>
         salt '*' pkg.purge pkgs='["foo", "bar"]'
     '''
-    return remove(name=name, pkgs=pkgs)
+    return remove(name=name, pkgs=pkgs, purge=True)
+
+
+def upgrade_available(name):
+    '''
+    Check whether or not an upgrade is available for a given package
+
+    CLI Example:
+
+    .. code-block:: bash
+
+        salt '*' pkg.upgrade_available <package name>
+    '''
+    return latest_version(name) != ''
+
+
+def upgrade(name=None,
+            pkgs=None,
+            **kwargs):
+    '''
+    Run a full package upgrade (``pkg_add -u``), or upgrade a specific package
+    if ``name`` or ``pkgs`` is provided.
+    ``name`` is ignored when ``pkgs`` is specified.
+
+    Returns a dictionary containing the changes:
+
+    .. code-block:: python
+
+        {'<package>': {'old': '<old-version>',
+                       'new': '<new-version>'}}
+
+
+    CLI Example:
+
+    .. code-block:: bash
+
+        salt '*' pkg.upgrade
+        salt '*' pkg.upgrade python%2.7
+    '''
+    old = list_pkgs()
+
+    cmd = ['pkg_add',  '-Ix', '-u']
+
+    if kwargs.get('noop', False):
+        cmd.append('-n')
+
+    if pkgs:
+        cmd.extend(pkgs)
+    elif name:
+        cmd.append(name)
+
+    # Now run the upgrade, compare the list of installed packages before and
+    # after and we have all the info we need.
+    result = __salt__['cmd.run_all'](cmd, output_loglevel='trace',
+                                     python_shell=False)
+
+    __context__.pop('pkg.list_pkgs', None)
+    new = list_pkgs()
+    ret = salt.utils.compare_dicts(old, new)
+
+    if result['retcode'] != 0:
+        raise CommandExecutionError(
+                'Problem encountered upgrading packages',
+                info={'changes': ret, 'result': result}
+        )
+
+    return ret
