@@ -16,14 +16,17 @@ Useful documentation:
 . https://github.com/xapi-project/xen-api/tree/master/scripts/examples/python
 . http://xenbits.xen.org/gitweb/?p=xen.git;a=tree;f=tools/python/xen/xm;hb=HEAD
 '''
+from __future__ import absolute_import
 
 # Import python libs
 import sys
 import contextlib
 import os
+from salt.ext.six.moves import range
+from salt.ext.six.moves import map
 
 try:
-    import importlib
+    import importlib  # pylint: disable=minimum-python-version
     HAS_IMPORTLIB = True
 except ImportError:
     # Python < 2.7 does not have importlib
@@ -32,6 +35,7 @@ except ImportError:
 # Import salt libs
 from salt.exceptions import CommandExecutionError
 import salt.utils
+import salt.modules.cmdmod
 
 # Define the module's virtual name
 __virtualname__ = 'virt'
@@ -63,7 +67,7 @@ def _check_xenapi():
 def __virtual__():
     if _check_xenapi() is not False:
         return __virtualname__
-    return False
+    return (False, "Module xapi: xenapi check failed")
 
 
 @contextlib.contextmanager
@@ -170,7 +174,7 @@ def _get_val(record, keys):
     return data
 
 
-def list_vms():
+def list_domains():
     '''
     Return a list of virtual machine names on the minion
 
@@ -178,7 +182,7 @@ def list_vms():
 
     .. code-block:: bash
 
-        salt '*' virt.list_vms
+        salt '*' virt.list_domains
     '''
     with _get_xapi_session() as xapi:
         hosts = xapi.VM.get_all()
@@ -227,7 +231,7 @@ def vm_info(vm_=None):
             if ret is not None:
                 info[vm_] = ret
         else:
-            for vm_ in list_vms():
+            for vm_ in list_domains():
                 ret = _info(vm_)
                 if ret is not None:
                     info[vm_] = _info(vm_)
@@ -254,7 +258,7 @@ def vm_state(vm_=None):
             info[vm_] = _get_record_by_label(xapi, 'VM', vm_)['power_state']
             return info
 
-        for vm_ in list_vms():
+        for vm_ in list_domains():
             info[vm_] = _get_record_by_label(xapi, 'VM', vm_)['power_state']
         return info
 
@@ -630,35 +634,21 @@ def resume(vm_):
             return False
 
 
-# FIXME / TODO
-# This function does NOT use the XenAPI. Instead, it use good old xm / xl.
-# On Xen Source, creating a virtual machine using XenAPI is really painful.
-# XCP / XS make it really easy using xapi.Async.VM.start, but I don't use
-# those on any of my networks.
-def create(config_):
+def start(config_):
     '''
     Start a defined domain
 
     CLI Example:
 
-    .. code-block:: bash
-
-        salt '*' virt.create <path to Xen cfg file>
-    '''
-    return __salt__['cmd.run']('{0} create {1}'.format(_get_xtool(), config_), python_shell=False)
-
-
-def start(config_):
-    '''
-    Alias for the obscurely named 'create' function
-
-    CLI Example:
-
     .. code-block:: bash
 
         salt '*' virt.start <path to Xen cfg file>
     '''
-    return create(config_)
+    # FIXME / TODO
+    # This function does NOT use the XenAPI. Instead, it use good old xm / xl.
+    # On Xen Source, creating a virtual machine using XenAPI is really painful.
+    # XCP / XS make it really easy using xapi.Async.VM.start instead. Anyone?
+    return __salt__['cmd.run']('{0} create {1}'.format(_get_xtool(), config_), python_shell=False)
 
 
 def reboot(vm_):
@@ -744,7 +734,7 @@ def migrate(vm_, target,
             return False
 
 
-def destroy(vm_):
+def stop(vm_):
     '''
     Hard power down the virtual machine, this is equivalent to pulling the
     power
@@ -753,7 +743,7 @@ def destroy(vm_):
 
     .. code-block:: bash
 
-        salt '*' virt.destroy <vm name>
+        salt '*' virt.stop <vm name>
     '''
     with _get_xapi_session() as xapi:
         vm_uuid = _get_label_uuid(xapi, 'VM', vm_)
@@ -783,9 +773,10 @@ def is_hyper():
         # virtual_subtype isn't set everywhere.
         return False
     try:
-        if 'xen_' not in salt.utils.fopen('/proc/modules').read():
-            return False
-    except IOError:
+        with salt.utils.fopen('/proc/modules') as fp_:
+            if 'xen_' not in fp_.read():
+                return False
+    except (OSError, IOError):
         return False
     # there must be a smarter way...
     return 'xenstore' in __salt__['cmd.run'](__grains__['ps'])
@@ -835,7 +826,7 @@ def vm_cputime(vm_=None):
             info[vm_] = _info(vm_)
             return info
 
-        for vm_ in list_vms():
+        for vm_ in list_domains():
             info[vm_] = _info(vm_)
 
         return info
@@ -885,7 +876,7 @@ def vm_netstats(vm_=None):
         if vm_:
             info[vm_] = _info(vm_)
         else:
-            for vm_ in list_vms():
+            for vm_ in list_domains():
                 info[vm_] = _info(vm_)
         return info
 
@@ -932,6 +923,6 @@ def vm_diskstats(vm_=None):
         if vm_:
             info[vm_] = _info(vm_)
         else:
-            for vm_ in list_vms():
+            for vm_ in list_domains():
                 info[vm_] = _info(vm_)
         return info
