@@ -10,13 +10,17 @@ Perform an HTTP query and statefully return the result
 # Import python libs
 from __future__ import absolute_import
 import re
+import logging
+import time
 
 __monitor__ = [
         'query',
         ]
 
+log = logging.getLogger(__name__)
 
-def query(name, match=None, match_type='string', status=None, **kwargs):
+
+def query(name, match=None, match_type='string', status=None, wait_for=None, **kwargs):
     '''
     Perform an HTTP query and statefully return the result
 
@@ -58,7 +62,7 @@ def query(name, match=None, match_type='string', status=None, **kwargs):
         query_example:
           http.query:
             - name: 'http://example.com/'
-            - status: '200'
+            - status: 200
 
     '''
     # Monitoring state, but changes may be made over HTTP
@@ -82,7 +86,10 @@ def query(name, match=None, match_type='string', status=None, **kwargs):
     if __opts__['test']:
         kwargs['test'] = True
 
-    data = __salt__['http.query'](name, **kwargs)
+    if wait_for:
+        data = __salt__['http.wait_for_successful_query'](name, wait_for=wait_for, **kwargs)
+    else:
+        data = __salt__['http.query'](name, **kwargs)
 
     if match is not None:
         if match_type == 'string':
@@ -93,7 +100,7 @@ def query(name, match=None, match_type='string', status=None, **kwargs):
                 ret['result'] = False
                 ret['comment'] += ' Match text "{0}" was not found.'.format(match)
         elif match_type == 'pcre':
-            if re.search(match, data['text']):
+            if re.search(match, data.get('text', '')):
                 ret['result'] = True
                 ret['comment'] += ' Match pattern "{0}" was found.'.format(match)
             else:
@@ -118,3 +125,45 @@ def query(name, match=None, match_type='string', status=None, **kwargs):
 
     ret['data'] = data
     return ret
+
+
+def wait_for_successful_query(name, wait_for=300, **kwargs):
+    '''
+    Like query but, repeat and wait until match/match_type or status is fulfilled. State returns result from last
+    query state in case of success or if no successful query was made within wait_for timeout.
+
+    name
+        The name of the query.
+
+    wait_for
+        Total time to wait for requests that succeed.
+
+    request_interval
+        Optional interval to delay requests by N seconds to reduce the number of requests sent.
+
+    .. note::
+
+        All other arguements are passed to the http.query state.
+    '''
+    starttime = time.time()
+
+    while True:
+        caught_exception = None
+        ret = None
+        try:
+            ret = query(name, **kwargs)
+            if ret['result']:
+                return ret
+        except Exception as exc:
+            caught_exception = exc
+
+        if time.time() > starttime + wait_for:
+            if not ret and caught_exception:
+                # workaround pylint bug https://www.logilab.org/ticket/3207
+                raise caught_exception  # pylint: disable=E0702
+            return ret
+        else:
+            # Space requests out by delaying for an interval
+            if 'request_interval' in kwargs:
+                log.debug("delaying query for {0} seconds.".format(kwargs['request_interval']))
+                time.sleep(kwargs['request_interval'])
