@@ -14,13 +14,19 @@ import binascii
 import salt.crypt
 import salt.payload
 import salt.master
+import salt.transport.frame
 import salt.utils.event
+import salt.ext.six as six
 from salt.utils.cache import CacheCli
 
 # Import Third Party Libs
 import tornado.gen
-from Crypto.Cipher import PKCS1_OAEP
-from Crypto.PublicKey import RSA
+try:
+    from Cryptodome.Cipher import PKCS1_OAEP
+    from Cryptodome.PublicKey import RSA
+except ImportError:
+    from Crypto.Cipher import PKCS1_OAEP
+    from Crypto.PublicKey import RSA
 
 
 log = logging.getLogger(__name__)
@@ -29,7 +35,10 @@ log = logging.getLogger(__name__)
 # TODO: rename
 class AESPubClientMixin(object):
     def _verify_master_signature(self, payload):
-        if payload.get('sig') and self.opts.get('sign_pub_messages'):
+        if self.opts.get('sign_pub_messages'):
+            if not payload.get('sig', False):
+                raise salt.crypt.AuthenticationError('Message signing is enabled but the payload has no signature.')
+
             # Verify that the signature is valid
             master_pubkey_path = os.path.join(self.opts['pki_dir'], 'minion_master.pub')
             if not salt.crypt.verify_signature(master_pubkey_path, payload['load'], payload.get('sig')):
@@ -66,7 +75,7 @@ class AESReqServerMixin(object):
             # cases, 'aes' is already set in the secrets.
             salt.master.SMaster.secrets['aes'] = {
                 'secret': multiprocessing.Array(ctypes.c_char,
-                              salt.crypt.Crypticle.generate_key_string()),
+                              six.b(salt.crypt.Crypticle.generate_key_string())),
                 'reload': salt.crypt.Crypticle.generate_key_string
             }
 
@@ -108,11 +117,14 @@ class AESReqServerMixin(object):
             return self.crypticle.dumps({})
         except IOError:
             log.error('AES key not found')
-            return 'AES key not found'
+            return {'error': 'AES key not found'}
 
         pret = {}
         cipher = PKCS1_OAEP.new(pub)
-        pret['key'] = cipher.encrypt(key)
+        if six.PY2:
+            pret['key'] = cipher.encrypt(key)
+        else:
+            pret['key'] = cipher.encrypt(salt.utils.to_bytes(key))
         pret[dictkey] = pcrypt.dumps(
             ret if ret is not False else {}
         )
@@ -189,7 +201,8 @@ class AESReqServerMixin(object):
                              'id': load['id'],
                              'pub': load['pub']}
 
-                    self.event.fire_event(eload, salt.utils.event.tagify(prefix='auth'))
+                    if self.opts.get('auth_events') is True:
+                        self.event.fire_event(eload, salt.utils.event.tagify(prefix='auth'))
                     return {'enc': 'clear',
                             'load': {'ret': 'full'}}
 
@@ -220,27 +233,31 @@ class AESReqServerMixin(object):
             eload = {'result': False,
                      'id': load['id'],
                      'pub': load['pub']}
-            self.event.fire_event(eload, salt.utils.event.tagify(prefix='auth'))
+            if self.opts.get('auth_events') is True:
+                self.event.fire_event(eload, salt.utils.event.tagify(prefix='auth'))
             return {'enc': 'clear',
                     'load': {'ret': False}}
 
         elif os.path.isfile(pubfn):
             # The key has been accepted, check it
-            if salt.utils.fopen(pubfn, 'r').read().strip() != load['pub'].strip():
-                log.error(
-                    'Authentication attempt from {id} failed, the public '
-                    'keys did not match. This may be an attempt to compromise '
-                    'the Salt cluster.'.format(**load)
-                )
-                # put denied minion key into minions_denied
-                with salt.utils.fopen(pubfn_denied, 'w+') as fp_:
-                    fp_.write(load['pub'])
-                eload = {'result': False,
-                         'id': load['id'],
-                         'pub': load['pub']}
-                self.event.fire_event(eload, salt.utils.event.tagify(prefix='auth'))
-                return {'enc': 'clear',
-                        'load': {'ret': False}}
+            with salt.utils.fopen(pubfn, 'r') as pubfn_handle:
+                if pubfn_handle.read().strip() != load['pub'].strip():
+                    log.error(
+                        'Authentication attempt from {id} failed, the public '
+                        'keys did not match. This may be an attempt to compromise '
+                        'the Salt cluster.'.format(**load)
+                    )
+                    # put denied minion key into minions_denied
+                    with salt.utils.fopen(pubfn_denied, 'w+') as fp_:
+                        fp_.write(load['pub'])
+                    eload = {'result': False,
+                             'id': load['id'],
+                             'act': 'denied',
+                             'pub': load['pub']}
+                    if self.opts.get('auth_events') is True:
+                        self.event.fire_event(eload, salt.utils.event.tagify(prefix='auth'))
+                    return {'enc': 'clear',
+                            'load': {'ret': False}}
 
         elif not os.path.isfile(pubfn_pend):
             # The key has not been accepted, this is a new minion
@@ -252,7 +269,8 @@ class AESReqServerMixin(object):
                 eload = {'result': False,
                          'id': load['id'],
                          'pub': load['pub']}
-                self.event.fire_event(eload, salt.utils.event.tagify(prefix='auth'))
+                if self.opts.get('auth_events') is True:
+                    self.event.fire_event(eload, salt.utils.event.tagify(prefix='auth'))
                 return {'enc': 'clear',
                         'load': {'ret': False}}
 
@@ -283,7 +301,8 @@ class AESReqServerMixin(object):
                          'act': key_act,
                          'id': load['id'],
                          'pub': load['pub']}
-                self.event.fire_event(eload, salt.utils.event.tagify(prefix='auth'))
+                if self.opts.get('auth_events') is True:
+                    self.event.fire_event(eload, salt.utils.event.tagify(prefix='auth'))
                 return ret
 
         elif os.path.isfile(pubfn_pend):
@@ -304,7 +323,8 @@ class AESReqServerMixin(object):
                          'act': 'reject',
                          'id': load['id'],
                          'pub': load['pub']}
-                self.event.fire_event(eload, salt.utils.event.tagify(prefix='auth'))
+                if self.opts.get('auth_events') is True:
+                    self.event.fire_event(eload, salt.utils.event.tagify(prefix='auth'))
                 return ret
 
             elif not auto_sign:
@@ -312,66 +332,73 @@ class AESReqServerMixin(object):
                 # Check if the keys are the same and error out if this is the
                 # case. Otherwise log the fact that the minion is still
                 # pending.
-                if salt.utils.fopen(pubfn_pend, 'r').read() != load['pub']:
-                    log.error(
-                        'Authentication attempt from {id} failed, the public '
-                        'key in pending did not match. This may be an '
-                        'attempt to compromise the Salt cluster.'
-                        .format(**load)
-                    )
-                    # put denied minion key into minions_denied
-                    with salt.utils.fopen(pubfn_denied, 'w+') as fp_:
-                        fp_.write(load['pub'])
-                    eload = {'result': False,
-                             'id': load['id'],
-                             'pub': load['pub']}
-                    self.event.fire_event(eload, salt.utils.event.tagify(prefix='auth'))
-                    return {'enc': 'clear',
-                            'load': {'ret': False}}
-                else:
-                    log.info(
-                        'Authentication failed from host {id}, the key is in '
-                        'pending and needs to be accepted with salt-key '
-                        '-a {id}'.format(**load)
-                    )
-                    eload = {'result': True,
-                             'act': 'pend',
-                             'id': load['id'],
-                             'pub': load['pub']}
-                    self.event.fire_event(eload, salt.utils.event.tagify(prefix='auth'))
-                    return {'enc': 'clear',
-                            'load': {'ret': True}}
+                with salt.utils.fopen(pubfn_pend, 'r') as pubfn_handle:
+                    if pubfn_handle.read() != load['pub']:
+                        log.error(
+                            'Authentication attempt from {id} failed, the public '
+                            'key in pending did not match. This may be an '
+                            'attempt to compromise the Salt cluster.'
+                            .format(**load)
+                        )
+                        # put denied minion key into minions_denied
+                        with salt.utils.fopen(pubfn_denied, 'w+') as fp_:
+                            fp_.write(load['pub'])
+                        eload = {'result': False,
+                                 'id': load['id'],
+                                 'act': 'denied',
+                                 'pub': load['pub']}
+                        if self.opts.get('auth_events') is True:
+                            self.event.fire_event(eload, salt.utils.event.tagify(prefix='auth'))
+                        return {'enc': 'clear',
+                                'load': {'ret': False}}
+                    else:
+                        log.info(
+                            'Authentication failed from host {id}, the key is in '
+                            'pending and needs to be accepted with salt-key '
+                            '-a {id}'.format(**load)
+                        )
+                        eload = {'result': True,
+                                 'act': 'pend',
+                                 'id': load['id'],
+                                 'pub': load['pub']}
+                        if self.opts.get('auth_events') is True:
+                            self.event.fire_event(eload, salt.utils.event.tagify(prefix='auth'))
+                        return {'enc': 'clear',
+                                'load': {'ret': True}}
             else:
                 # This key is in pending and has been configured to be
                 # auto-signed. Check to see if it is the same key, and if
                 # so, pass on doing anything here, and let it get automatically
                 # accepted below.
-                if salt.utils.fopen(pubfn_pend, 'r').read() != load['pub']:
-                    log.error(
-                        'Authentication attempt from {id} failed, the public '
-                        'keys in pending did not match. This may be an '
-                        'attempt to compromise the Salt cluster.'
-                        .format(**load)
-                    )
-                    # put denied minion key into minions_denied
-                    with salt.utils.fopen(pubfn_denied, 'w+') as fp_:
-                        fp_.write(load['pub'])
-                    eload = {'result': False,
-                             'id': load['id'],
-                             'pub': load['pub']}
-                    self.event.fire_event(eload, salt.utils.event.tagify(prefix='auth'))
-                    return {'enc': 'clear',
-                            'load': {'ret': False}}
-                else:
-                    pass
+                with salt.utils.fopen(pubfn_pend, 'r') as pubfn_handle:
+                    if pubfn_handle.read() != load['pub']:
+                        log.error(
+                            'Authentication attempt from {id} failed, the public '
+                            'keys in pending did not match. This may be an '
+                            'attempt to compromise the Salt cluster.'
+                            .format(**load)
+                        )
+                        # put denied minion key into minions_denied
+                        with salt.utils.fopen(pubfn_denied, 'w+') as fp_:
+                            fp_.write(load['pub'])
+                        eload = {'result': False,
+                                 'id': load['id'],
+                                 'pub': load['pub']}
+                        if self.opts.get('auth_events') is True:
+                            self.event.fire_event(eload, salt.utils.event.tagify(prefix='auth'))
+                        return {'enc': 'clear',
+                                'load': {'ret': False}}
+                    else:
+                        os.remove(pubfn_pend)
 
         else:
             # Something happened that I have not accounted for, FAIL!
-            log.warn('Unaccounted for authentication failure')
+            log.warning('Unaccounted for authentication failure')
             eload = {'result': False,
                      'id': load['id'],
                      'pub': load['pub']}
-            self.event.fire_event(eload, salt.utils.event.tagify(prefix='auth'))
+            if self.opts.get('auth_events') is True:
+                self.event.fire_event(eload, salt.utils.event.tagify(prefix='auth'))
             return {'enc': 'clear',
                     'load': {'ret': False}}
 
@@ -412,8 +439,8 @@ class AESReqServerMixin(object):
                'pub_key': self.master_key.get_pub_str(),
                'publish_port': self.opts['publish_port']}
 
-        # sign the masters pubkey (if enabled) before it is
-        # send to the minion that was just authenticated
+        # sign the master's pubkey (if enabled) before it is
+        # sent to the minion that was just authenticated
         if self.opts['master_sign_pubkey']:
             # append the pre-computed signature to the auth-reply
             if self.master_key.pubkey_signature():
@@ -461,5 +488,6 @@ class AESReqServerMixin(object):
                  'act': 'accept',
                  'id': load['id'],
                  'pub': load['pub']}
-        self.event.fire_event(eload, salt.utils.event.tagify(prefix='auth'))
+        if self.opts.get('auth_events') is True:
+            self.event.fire_event(eload, salt.utils.event.tagify(prefix='auth'))
         return ret
