@@ -43,9 +43,8 @@ except ImportError:
 
 try:
     import backports_abc
-    HAS_BACKPORTS_ABC = True
 except ImportError:
-    HAS_BACKPORTS_ABC = False
+    import salt.ext.backports_abc as backports_abc
 
 try:
     import markupsafe
@@ -108,6 +107,7 @@ def get_tops(extra_mods='', so_mods=''):
             ]
 
     tops.append(_six.__file__.replace('.pyc', '.py'))
+    tops.append(backports_abc.__file__.replace('.pyc', '.py'))
 
     if HAS_CERTIFI:
         tops.append(os.path.dirname(certifi.__file__))
@@ -118,9 +118,6 @@ def get_tops(extra_mods='', so_mods=''):
     if HAS_SINGLEDISPATCH_HELPERS:
         tops.append(singledispatch_helpers.__file__.replace('.pyc', '.py'))
 
-    if HAS_BACKPORTS_ABC:
-        tops.append(backports_abc.__file__.replace('.pyc', '.py'))
-
     if HAS_SSL_MATCH_HOSTNAME:
         tops.append(os.path.dirname(os.path.dirname(ssl_match_hostname.__file__)))
 
@@ -537,6 +534,7 @@ def gen_min(cachedir, extra_mods='', overwrite=False, so_mods='',
         'salt/modules/test.py',
         'salt/modules/selinux.py',
         'salt/modules/cmdmod.py',
+        'salt/modules/saltutil.py',
         'salt/minion.py',
         'salt/pillar',
         'salt/pillar/__init__.py',
